.MODULE biquad_sub;
{
        Cascaded Biquad IIR Filter Subroutine

        Calling Parameters
                SR1 = input sample
                I0 --> delay line buffer
                L0 = 0
                I1 --> list of scale factors for each biquad section
		L1 = 0 (in the case of a single biquad)
                L1 = number of biquad sections
                I4 --> scaled coefficients b2,b1,b0,a2,a1,  b2,b1,b0,a2,a1, ...
                L4 = 2.5 * filter order --or-- 5 * number of biquad sections
                M0,M4 = 1
                M1 = -3
                M2 = 1	(in the case of multiple biquads)
		M2 = 0	(in the case of a single biquad)
		M3 = (1 - length of delay line buffer)
                CNTR = number of biquad sections

        Return Values
                SR1 = output sample
                I0 --> inside delay line buffer
                I1 --> top of scale factor list
                I4 --> top of coefficients

        Altered Registers
                MX0,MX1,MY0,MR,SE,SR

        Computation Time 
               	ADSP-2101/2102:	 8 * number of biquad sections + 5 + 5 cycles
		ADSP-2100/2100A: 8 * number of biquad sections + 5 cycles	

        All coefficients and data values are assumed to be in 1.15 format.
}

.CONST  N = 3;             { number of biquad sections, example: 3 }
.CONST  N_x_5 = 15;        { number of biquad sections times five }

.VAR/DM delayline[8];      { (4 + (N-1)*2) where N=number of biquad sections}
.VAR/DM scalelist[N];      { initialize with the scale factor for each biquad }
.VAR/PM coefflist[N_x_5];  { init with filter coefficients for each biquad }

.ENTRY  biquad;

biquad: I0=^delayline;
        DO sections UNTIL CE;
            SE=DM(I1,M2);
            MX0=DM(I0,M0), MY0=PM(I4,M4);                    {get x(n-2), b2}
            MR=MX0*MY0(SS), MX1=DM(I0,M0), MY0=PM(I4,M4);    {get x(n-1), b1}
            MR=MR+MX1*MY0(SS), MY0=PM(I4,M4);                {get         b0}
            MR=MR+SR1*MY0(SS), MX0=DM(I0,M0), MY0=PM(I4,M4); {get y(n-2), a2}
            MR=MR+MX0*MY0(SS), MX0=DM(I0,M1), MY0=PM(I4,M4); {get y(n-1), a1}
            DM(I0,M0)=MX1, MR=MR+MX0*MY0(RND);   {store x(n-1) as new x(n-2)}
sections:   DM(I0,M0)=SR1, SR=ASHIFT MR1 (HI);     {store x(n) as new x(n-1)}
        DM(I0,M0)=MX0;
        DM(I0,M3)=SR1;
        RTS;
.ENDMOD;
