{
	Dec2stagebuf.dsp
Real time Direct Form Finite Impulse Filter, N1,N2 taps, uses two cascaded
stages to decimate by M1 and M2 for a decrease of 1/(M1*M2) times the input 
sample rate. Uses an input buffer to increase efficiency.

	Input: adc.
	Output: dac.
}

.module/ram/abs=0	dec2stagebuf;
.const 			N1=32;
.const 			N2=148;
.const			M_1=2;			{decimate by factor of M}
.const			M_2=2;			{decimate by factor of M}
.const			M1xM2=4;		{(M_1 * M_2)		}
.const			M1xM2x2=8;		{(M_1 * M_2 * 2)	}
.var/pm/ram/circ	coef1[N1];
.var/pm/ram/circ	coef2[N2];
.var/dm/ram/circ	data1[N1];
.var/dm/ram/circ	data2[N2];
.var/dm/ram/circ	input_buf[m1xm2x2];
.var/dm/ram		input_count;
.port			adc;
.port			dac;
.init			coef1:<coef1.dat>;
.init			coef2:<coef2.dat>;
		rti;				{interrupt 0}
		rti;				{interrupt 1}
		rti;				{interrupt 2}
		jump sample;			{interrupt 3 at input rate}

initialize:	imask=b#0000;			{disable all interrupts}
		icntl=b#01111;			{edge sensitive interrupts}
		si=M1xM2;
		dm(input_count)=si;
		i4=^coef1;			{setup a circular buffer in PM}
		l4=%coef1;			
		m4=1;				{modifier for coef is 1}
		i5=^coef2;			{setup a circular buffer in PM}
		l5=%coef2;			
		i0=^data1;			{setup a circular buffer in DM}
		l0=%data1;			
		m0=1;				{modifier for data is 1}
		i1=^data2;			{setup a circular buffer in DM}
		l1=%data2;
		i2=^input_buf;			{setup input buffer in DM}
		l2=%input_buf;
		i3=^input_buf;			{setup second in buffer in DM}
		l3=%input_buf;
		imask=b#1000;			{enable interrupt 3}
{}
wait_full:	ax0=dm(input_count);		{wait till input buffer is full}
		ar=pass ax0;
		if ne jump wait_full;

		ar= M1xM2;			{reinitialize input counter}
		dm(input_count)=ar;
		cntr=M_2;
		do stage_1 until ce;
		  cntr=M_1;
		  do dump_buf until ce;
		    ar=dm(i3,m0);
dump_buf:	    dm(i0,m0)=ar;
		  cntr=N1 - 1;
		  mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
		  do taploop1 until ce;    	{N-1 taps of FIR}
taploop1:	    mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
		  mr=mr+mx0*my0(rnd);		{last tap with round}
		  if mv sat mr;			{saturate result if overflow}
stage_1:	  dm(i1,m0)=mr1;		{pass to next filter stage}

		cntr=N2 - 1;
		mr=0, mx0=dm(i1,m0), my0=pm(i5,m4);
		do taploop2 until ce;    	{N-1 taps of FIR}
taploop2:	   mr=mr+mx0*my0(ss), mx0=dm(i1,m0), my0=pm(i5,m4);
		mr=mr+mx0*my0(rnd);		{last tap with round}
		if mv sat mr;			{saturate result if overflow}
		dm(dac)=mr1;			{output data sample}
		jump wait_full;

{___________________interrupt code occurs at the sample rate__________________}
sample:		ena sec_reg;			{context save}    
		ay0=dm(adc);			{input data sample}
		dm(i2,m0)=ay0;			{load input buffer}
		ay0=dm(input_count);		
		ar=ay0-1;			{decrement and update counter}
		dm(input_count)=ar;
		rti;
.endmod;
