{
	Dec_eff.dsp
Real time Direct Form Finite Impulse Filter, N taps, decimates by M for a
decrease of 1/M times the input sample rate. This version uses an input buffer
to all the filter computations to occur in parrallel with inputs. This allows
larger order filter for a given input sample rate. to save time an index 
register is used for the input buffer as well as for a decimation counter.

	Input: adc.
	Output: dac.
}

.module/ram/abs=0	eff_decimate;
.const 			N=300;
.const			M=4;			{decimate by factor of M}
.var/pm/ram/circ	coef[N];
.var/dm/ram/circ	data[N];
.var/dm/ram/circ	input_buf[M];
.port			adc;
.port			dac;
.init			coef:<coef.dat>;
		rti;				{interrupt 0}
		rti;				{interrupt 1}
		rti;				{interrupt 2}
		jump sample;			{interrupt 3= input sample rate}

initialize:	imask=b#0000;			{disable all interrupts}
		icntl=b#01111;			{edge sensitive interrupts}
		i4=^coef;			{setup a circular buffer in PM}
		l4=%coef;			
		m4=1;				{modifier for coef is 1}
		i0=^data;			{setup a circular buffer in DM}
		l0=%data;			
		m0=1;				{modifier for data is 1}
		i1=^input_buf;			{setup a circular buffer in DM}
		l1=%input_buf;			
		imask=b#1000;			{enable interrupt 3}
{}
wait_M:		ax0=i1;				{wait for M inputs}
		ay0=^input_buf;
		ar=ax0-ay0;			{test if pointer is at start}
		if ne jump wait_M;

{__________________code below occurs at 1/M times the sample rate______________}
		cntr=M;
		do load_data until ce;
		   ar=dm(i1,m0);
load_data:	   dm(i0,m0)=ar;

fir:		cntr=N - 1;
		mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
		do taploop until ce;    	{N-1 taps of FIR}
taploop:	   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
		mr=mr+mx0*my0(rnd);		{last tap with round}
		if mv sat mr;			{saturate result if overflow}
		dm(dac)=mr1;			{output data sample}

wait_again:	ax0=i1;
		ay0=^input_buf;
		ar=ax0-ay0;			{test and wait if i1 still}
		if eq jump wait_again;		{points to start of input_buf}
		jump wait_M;				

{___________________sample input, code occurs at the sample rate_______________}
sample:		ena sec_reg;			{so no registers will get lost}
		ay0=dm(adc);			{get input sample}
		dm(i1,m0)=ay0;			{load in M long buffer}
		rti;
.endmod;
