{
	Int2stage.dsp
Two stage cascaded real time Direct Form Finite Impulse Filter, N1, N2 taps,
uses an efficient algorithm to interpolate by L1*L2 for an increase of L1*L2 
times the input sample rate. A restriction on the number of taps is that N 
divided by L be an integer.

	Input: adc.
	Output: dac.
}

.module/ram/abs=0	interpolate_2stage;
.const 			N1=32;
.const 			N2=148;
.const			L_1=2;			{stage one factor is L1}
.const			L_2=2;			{stage two factor is L2}
.const			N1overL1=16;
.const			N2overL2=74;
.var/pm/ram/circ	coef1[N1];
.var/pm/ram/circ	coef2[N2];
.var/dm/ram/circ	data1[N1overL1];
.var/dm/ram/circ	data2[N2overL2];
.var/dm/ram		counter1;
.var/dm/ram		counter2;
.port			adc;
.port			dac;
.init			coef1:<coef1.dat>;
.init			coef2:<coef2.dat>;
		rti;				{interrupt 0}
		rti;				{interrupt 1}
		rti;				{interrupt 1}
		jump sample;			{interrupt 3= L1*L2 output rate}

initialize:	imask=b#0000;			{disable all interrupts}
		icntl=b#01111;			{edge sensitive interrupts}
		si=1;				{set interpolate counters to 1}
		dm(counter1)=si;		{for first data sample}
		dm(counter2)=si;
		i4=^coef1;			{setup a circular buffer in PM}
		l4=%coef1;			
		m4=L_1;				{modifier for coef is L1}
		m6=-1;				{modifier to shift coef back -1}
		i5=^coef2;			{setup a circular buffer in PM}
		l5=%coef2;			
		m5=L_2;				{modifier for coef is L2}
		i0=^data1;			{setup a circular buffer in DM}
		l0=%data1;
		m0=1;
		i1=^data2;			{setup a circular buffer in DM}
		l1=%data2;			
		imask=b#1000;			{enable interrupt 3}
wait_interrupt:	jump wait_interrupt;		{infinite wait loop}

{}
{_____________________________Interpolate______________________________________}
sample:		modify(i5,m6);			{shifts coef pointer back by -1}
		ay0=dm(counter2);
		ar=ay0-1;			{decrement and update counter}
		dm(counter2)=ar;
		if ne jump do_fir2;		{test, do stage 1 if L_2 times}

		modify(i4,m6);			{shifts coef pointer back by -1}
		ay0=dm(counter1);
		ar=ay0-1;			{decrement and update counter.}
		dm(counter1)=ar;
		if ne jump do_fir1;		{test and input if L_1 times}

{___________________input data sample, code occurs at the input sample rate____}
do_input:	ay0=dm(adc);			{input data sample}
		dm(i0,m0)=ay0;			{update delay line with newest}
		modify(i4,m4);			{shifts coef1 pointer by L1}
		dm(counter1)=m4;		{reset counter1}

{__________________filter pass, occurs at L1 times the input sample rate_______}
do_fir1:	cntr=N1overL1 - 1;		{N1/L1-1 because round last tap}
		mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
		do taploop1 until ce;    	{N1/L_1-1 taps of FIR}
taploop1:	   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
		mr=mr+mx0*my0(rnd);		{last tap with round}
		if mv sat mr;			{saturate result if overflow}
		dm(i1,m0)=mr1;			{update delay line with newest}
		modify(i5,m5);			{shifts coef2 pointer by L2}
		dm(counter2)=m5;		{reset counter2}

{__________________filter pass, occurs at (L1*L2) times the input sample rate__}
do_fir2	:	cntr=N2overL2 - 1;		{N2/L2-1 because round last tap}
		mr=0, mx0=dm(i1,m0), my0=pm(i5,m5);
		do taploop2 until ce;    	{N2/L_2-1 taps of FIR}
taploop2:	   mr=mr+mx0*my0(ss), mx0=dm(i1,m0), my0=pm(i5,m5);
		mr=mr+mx0*my0(rnd);		{last tap with round}
		if mv sat mr;			{saturate result if overflow}
		dm(dac)=mr1;			{output sample}
		rti;
.endmod;
