{
	Interpolate.dsp
Real time Direct Form Finite Impulse Filter, N taps, uses an efficient algorithm
to interpolate by L for an increase of L times the input sample rate. A 
restriction on the number of taps is that N divided by L be an integer.

	Input:adc.
	Output: dac.
}

.module/ram/abs=0	interpolate;
.const 			N=300;
.const			L=4;			{interpolate by factor of L}
.const			NoverL=75;
.var/pm/ram/circ	coef[N];
.var/dm/ram/circ	data[NoverL];
.var/dm/ram		counter;
.port			adc;
.port			dac;
.init			coef:<coef.dat>;
		rti;				{interrupt 0}
		rti;				{interrupt 1}
		rti;				{interrupt 1}
		jump sample;			{interrupt 3 at (L*input rate)}

initialize:	imask=b#0000;			{disable all interrupts}
		icntl=b#01111;			{edge sensitive interrupts}
		si=1;				{set interpolate counter to 1}
		dm(counter)=si;			{for first data sample}
		i4=^coef;			{setup a circular buffer in PM}
		l4=%coef;			
		m4=L;				{modifier for coef is L!}
		m5=-1;				{modifier to shift coef back -1}
		i0=^data;			{setup a circular buffer in DM}
		l0=%data;			
		m0=1;		
		imask=b#1000;			{enable interrupt 3}
wait_interrupt:	jump wait_interrupt;		{infinite wait loop}

{}
{_____________________________Interpolate______________________________________}
sample:		modify(i4,m5);			{shifts coef pointer back by -1}
		ay0=dm(counter);		
		ar=ay0-1;			{decrement and update counter}
		dm(counter)=ar;			
		if ne jump do_fir;		{test and input if L times}

{___________________input data sample, code occurs at the sample rate__________}
do_input:	ay0=dm(adc);			{input data sample}
		dm(i0,m0)=ay0;			{update delay line with newest}
		modify(i4,m4);			{shifts coef pointer up by L}
		dm(counter)=m4;			{reset counter to L}

{__________________filter pass, occurs at L times the input sample rate________}
do_fir	:	cntr=NoverL - 1;		{N/L-1 since round on last tap} 
		mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
		do taploop until ce;    	{N/L-1 taps of FIR}
taploop:	   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
		mr=mr+mx0*my0(rnd);		{last tap with round}
		if mv sat mr;			{saturate result if overflowed}
		dm(dac)=mr1;			{output sample}
		rti;				
.endmod;
