.MODULE all_pole_lattice_filter;
 
{       All-Pole Lattice Filter Routine
 
        Calling Parameters
                CNTR = Length of Excitation Signal
                I0 --> Excitation Signal                L0 = 0
                I1 --> Coefficient Buffer (circular)    L1 = Filter Order
                I2 --> Output Buffer                    L2 = 0
                I4 --> Delay Line Buffer (circular)     L4 = Filter Order
                AR = Formatted 1
                M0,M4 = 1                               M1,M5 = -1
                M6 = 3                                  M7 = -2
                SE = Appropriate scale value
                AX0 = Filter Order - 1

        Return Values
                Output Buffer Filled

        Altered Registers
                MX0,MY0,MY1,MR,SR,I0,I1,I2,I4

        Computation Time
                (6*(Filter Length-1)+8) * Output Buffer Length + 3 + 6 cycles

}
 
.ENTRY  p_latt;

p_latt: MY0=DM(I0,M0);                                   {Get input data}
        DO outloop UNTIL CE;                             {Loop through output}
                CNTR=AX0;
                MR=AR*MY0 (SS), MX0=DM(I1,M0), MY0=PM(I4,M4);     {Get U,K}
                MR=MR-MX0*MY0 (SS), MX0=DM(I1,M0), MY0=PM(I4,M4); {MR=X10}
                DO dataloop UNTIL CE;                    {Loop through filter}
                        MR=MR-MX0*MY0 (SS);              {Compute Xz}
                        SR=ASHIFT MR1 (HI);              {Reformat Xz}
                        MY1=SR1, MR=AR*MY0 (SS);         {Format Uz+1}
                        MR=MR+MX0*MY1 (SS), MX0=DM(I1,M0),MY0=PM(I4,M7); {MR=Uz}
                        SR=ASHIFT MR1 (HI);              {Reformat Uz}
dataloop:               PM(I4,M6)=SR1, MR=AR*MY1 (SS);   {Save Uz format Xz}
                MY0=PM(I4,M7), MX0=DM(I1,M1);            {Reset Pointers}
                MY0=DM(I0,M0), SR=ASHIFT MR1 (HI);       {Get new data point}
                DM(I2,M0)=MY1, SR=SR OR LSHIFT MR0 (LO); {Store output}
outloop:        PM(I4,M4)=SR1;                           {Save Y}
        RTS;
.ENDMOD;
