{
	Ratio_buf.dsp
Real time Direct Form Finite Impulse Filter, N taps.  Efficient algorithm
to interpolate by L and decimate by M for a L/M change in the input sample
rate.  Uses an input buffer so that the filter computations to occur in
parrallel with inputing and outputing data.  This allows a larger number of
filter taps for a given input sample rate.

	Input: adc.
	Output: dac.
}

.module/ram/abs=0	Ratio_eff;
.const 			N=300;			{N taps, N coefficients}
.const 			L=3;			{decimate by factor of L}
.const 			NoverL=100;		{NoverL must be an integer}
.const			M=2;			{decimate by factor of M}
.const			intMoverL=2;		{smallest integer GE M/L}
.var/pm/ram/circ	coef[N];		{coefficient circular buffer}
.var/dm/ram/circ	data[NoverL];		{data delay line}
.var/dm/ram		input_buf[intMoverL];	{input buffer is not circular}
.var/dm/ram		countin;
.var/dm/ram		countout;
.var/dm/ram		out_flag;		{true when time to calc. output}
.port			adc;
.port			dac;
.init			coef:<coef.dat>;
		rti;				{interrupt 0}
		jump interrupt;			{interrupt 1= L * input rate}
		rti;				{interrupt 2}
		rti;				{interrupt 3}

initialize:	imask=b#0000;			{disable all interrupts}
		icntl=b#01111;			{edge sensitive interrupts}
		si=0;				{variables inititial conditions}
		dm(out_flag)=si;		{true if time to calc. output}
		si=1;
		dm(countin)=si;			{input every L interrupts}
		dm(countout)=si;		{output every M interrupts}
		i4=^coef;			{setup a circular buffer in PM}
		l4=%coef;			
		m4=L;				{modifier for coef buffer}
		i5=0;				{i5 tracks coefficient updates}
		l5=0;
		m5=-M;				{coef modify done each output}
		i0=^data;			{setup delay line in DM}
		l0=%data;			
		m0=1;				{modifier for data is 1}
		i1=^input_buf;			{setup input data buffer in DM}
		l1=0;				{input buffer is not circular}
		imask=b#0010;			{enable interrupt 3}
{}
{___________________wait for time to calculate an output sample________________}
wait_out:	ax0=dm(out_flag);
		ar=pass ax0;			{test if true}
		if eq jump wait_out;

{___________________code below occurs at the output sample rate________________}
		dm(out_flag)=l5;		{reset the output flag to 0}
		ax0=i1;				{calculate ammount in in buffer}
		ay0=^input_buf;
		ar=ax0-ay0;
		if eq jump modify_coef;		{skip do loop if buffer empty}
		cntr=ar;			{dump in buffer into delay line}
		i1=^input_buf;
		do load_data until ce;
		   ar=dm(i1,m0);
load_data:	   dm(i0,m0)=ar;
		i1=^input_buf;			{fix pointer to input_buf}

modify_coef:	modify(i5,m5);			{modify coef update by -M}
		m6=i5;
		modify(i4,m6);			{modify ^coef by coef update}
		i5=0;				{reset coef update}

fir:		cntr=NoverL-1;
		mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
		do taploop until ce;    	{N/L-1 taps of FIR}
taploop:	   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
		mr=mr+mx0*my0(rnd);		{last tap with round}
		if mv sat mr;			{saturate result if overflow}
		dm(dac)=mr1;			{output data sample}
		jump wait_out;

{___________________interrupt, code occurs at L times the input rate___________}
interrupt:	ena sec_reg;			{so no registers will get lost}
		ay0=dm(countin);		{test if time for input}
		ar=ay0-1;
		dm(countin)=ar;
		if ne jump skipin;

input:		ay0=dm(adc);			{get input sample}
		dm(i1,m0)=ay0;			{load in M long buffer}
		modify(i5,m4);			{modify the coef update by L}
		dm(countin)=m4;			{reset the input count to L}

skipin:		ay0=dm(countout);		{test if time for output}
		ar=ay0-1;
		dm(countout)=ar;
		if ne rti;

output:		dm(out_flag)=m0;		{set output flag to true 1}
		ar=M;				{reset output counter to M}
		dm(countout)=ar;
		rti;
.endmod;
